# trezormon

Система мониторинга (см. [документацию](https://trezor.myjetbrains.com/youtrack/articles/CLIENT-A-1))

## 1. Описание API

### Служебный API

|Endpoint|Метод|Ответ|Структура запроса|Описание|
|---|---|---|---|---|
|/service/update|POST|<p>400 - ошибка запроса;</p><p> 500 - ошибка обработки;</p><p> 204 - успешное выполнение</p>|MultipartForm - файл обновления в формате zip|Выполняет обновление СМ (только для macOS, linux)

### API работы с устройствами

|Endpoint|Метод|Ответ|Структура запроса|Описание|
|---|---|---|---|---|
|/devices/{id}|GET|500 - ошибка получения;<p>410 - устройство не найдено;</p>200 - устройство найдено. Структура - см. [устройства](#device_descr)|Vars: <p>id - идентификатор устройства;</p>|Возвращает устройство по id
|/devices/{id}|DELETE|500 - ошибка удаления;<p>204 -  устройство удалено</p>|Vars:<p>id - идентификатор устройства|Удаляет устройство
|/devices/{type}|POST|400 - некорректное тело;<p>500 - ошибка добавления/обновления</p><p>200 - успешное выполнение (в ответе - устройство)|{type} - тип устройства. См. [устройства](#device_descr). Тело - в зависимости от типа</p>|Добавляет или обновляет информацию об устройстве. Если устройство новое - id должен быть 0
|/devices/|GET|500 - ошибка получения<p>200 - устройства получены. Тело - массив устройств<p>|full - учитывать ли подустройства при выводе|Получает все подустройства
|/devices/{id}/subdevices/reload|GET|500 - ошибка загрузки<p>200 - успешное выполнение (в ответе - устройство)</p>|Vars: id - идентификатор устройства|Получает список подустройств из устройства
|/devices/{id}/service?enabled={bool}|GET|500 - ошибка переключения сервисного режима; 200 - состояние переключено, тело - переключаемое устройство|query: enabled - true=сервисный режим включен; false=выключен</p>|Переключает сервисный режим
|/devices/{id}/alarms/deactivate|GET|500 - ошибка переключения состояния тревог; 200 - состояние переключено, тело - переключаемое устройство|id - идентификатор устройства</p>|Выключает тревогу на устройстве (если включена)

### API работы с событиями

|Endpoint|Метод|Ответ|Структура запроса|Описание|
|---|---|---|---|---|
|/events/all?event_type={event_type}...&begin=0&end=0|GET|500 - ошибка получения событий<p>200 - события получены. Тело:<details><pre><p>[</p><p>    {</p><p>        "id": 49,</p><p>        "device\_id": 7,</p><p>        "state": "Error",</p><p>        "timestamp": "2021-02-15T00:02:33.994691+03:00",</p><p>        "message": "Device is in error state: Post http://home.panov.email/v1/login: net/http: request canceled while waiting for connection (Client.Timeout exceeded while awaiting headers)",</p><p>        "type": "Device state"</p><p>    },</p><p>    {</p><p>        "id": 47,</p><p>        "segment\_id": 2,</p><p>        "state": "Unknown",</p><p>        "timestamp": "2021-02-14T23:58:45.09362+03:00",</p><p>        "message": "",</p><p>        "type": "Segment state"</p><p>    },</p><p>    {</p><p>        "id": 48,</p><p>        "segment\_id": 5,</p><p>        "state": "Normal",</p><p>        "timestamp": "2021-02-14T23:58:45.093752+03:00",</p><p>        "message": "",</p><p>        "type": "Segment state"</p><p>    }</p><p>]</p></pre></details>|Query:<p>event_type - типы событий<p>begin - время начала отбора событий (по умолчанию - 0)</p><p>end - время конца отбора событий (по умолчанию - текущее время)</p>|Возвращает события по указанным интервалам и типам
|/events/remove/{event_id}|DELETE|500 - ошибка удаления<p>204 - событие удалено|Vars: event_id - идентификатор события|Удаляет событие
|/events/processed/{event_id}|POST|500 - ошибка установки события<p>204 - событие обратано|Vars: event_id - идентификатор события|Помечает событие как обработанное

### API работы с сегментами

|Endpoint|Метод|Ответ|Структура запроса|Описание|
|---|---|---|---|---|
|/segments/get/{segment\_id}|GET|400 - некорректный ID сегмента (segment\_id <= 0)<p>500 - ошибка получения данных (формат smerrors)<p>200 - сегмент найден. Тело: <details><pre><p>{</p><p>    "id": 2,</p><p>    "state": "Unknown",</p><p>    "name": "test\_segment",</p><p>    "description": "Тестовый сегмент проверка изменения #2",</p><p>    "devices": []</p><p>}</p></pre></details>|Vars: segment\_id - ID сегмента|Возвращает сегмент по ID
|/segments/devices/{segment\_id}/{device\_id}|POST|400 - некорректный ID сегмента или устройства (segment\_id <= 0 или device\_id <=0)<p>500 - ошибка добавления устройства (формат smerrors)<p>204 - устройство добавлено в сегмент.|segment\_id - ID сегмента; device\_id - идентификатор устройства|Добавляет указанное устройство в указанный сегмент
|/segments/devices/{segment\_id}/{device\_id}|DELETE|400 - некорректный ID сегмента или устройства (segment\_id <= 0 или device\_id <=0)<p>500 - ошибка удаления устройства (формат smerrors)<p>204 - устройство удалено из сегмента|segment\_id - ID сегмента<p>device\_id - ID сегмента|Удаляет указанное устройство и указанного сегмент
|/segments/remove/{segment\_id}|DELETE|400 - некорректный ID сегмента (segment\_id <= 0)<p>500 - ошибка удаления сегмента (формат smerrors)<p>200 - сегмент удален.|Vars: segment\_id - ID сегмента|Удаляет сегмент
|/segments/add|POST|400 - некорректное тело запроса<p>500 - ошибка добавления сегмента (формат smerrors)<p>200 - сегмент добавлен (в ответе - сегмент)|Body: сегмент<details><pre><p>{</p><p>    "id": 0,</p><p>    "name": "test\_segment",</p><p>    "description": "Тестовый сегмент",</p><p>    "devices": []</p><p>}</p></pre></details><p>|Добавляет или изменяет сегмент (для добавления, id в запросе = 0)
|/segments/all|GET|500 - ошибка получения данных (формат smerrors)<p>200 - сегменты найдены. Тело: <details><pre><p>[</p><p>    {</p><p>        "id": 2,</p><p>        "state": "Unknown",</p><p>        "name": "test\_segment",</p><p>        "description": "Тестовый сегмент проверка изменения #2",</p><p>        "devices": []</p><p>    },</p><p>    ...</p><p>]</p></pre></details>|-|Возвращает все сегменты

### API для работы с планами

|Endpoint|Метод|Ответ|Структура запроса|Описание|
|---|---|---|---|---|
|/plans/all|GET|500 - ошибка получения (smerrors)<p>200 - планы получены. Схема см. [план](#other_plan)|-| Возвращает все планы
|/plans/add|POST|400 - некорректный запрос<p>500 - ошибка добавления плана<p>200 - план добавлен (в теле - добавленный план, см. [план](#other_plan)|body - JSON-структура плана|Добавляет (или изменяет) план
|/plans/get/{plan\_id}|GET|500 - ошибка получения (smerrors)<p>200 - планы получены. Схема см. [план](#other_plan)|<p>plan_id - идентификатор нужного плана|Возвращает указанный план
|/plans/remove/{plan\_id}|DELETE|500 - ошибка получения (smerrors)<p>204 - план удален|plan\_id - идентификатор нужного плана|Удаляет указанный план
|/plans/images/upload/{plan\_id}|POST|400 - некорректный запрос<p>500 - ошибка добавления изображения<p>204 - изображение добавлено|plan_id - идентификатор требуемого плана<p>form file "background" - изображение плана|Добавляет изображение плана
|/plans/images/get/{image_name}|GET|404 - изображение не найдено<p>200 - изображение найдено|image_name - имя изображения (из структуры плана)|Возвращает изображение
|/plans/objects/remove/{plan_obj_id}|DELETE|500 - ошибка удаления объекта плана<p>204 - объект плана удален| plan_obj_id - объект плана|Удаляет объект плана

## 2. Описание конфигурации

### Настройки логгирования (секции `[log.*]`)

Доступные для конфигурирования секции: `api, http, db, monitoring, standartLogger (без указания подсекции)`

|Имя параметра|Тип|Обязательный|Описание|Значение по умолчанию
|---|---|---|---|---|
|stdout|bool|Нет|Вывод в stdout информации из логгера|true
|level|string|Нет|Уровень логгирования|INFO
|syslog_level|string|Нет|Уровень логгирования в syslog|INFO
|syslog|string|Адрес syslog-сервера (в формате ip/proto)|-

Используемый логгер - logrus, настройки уровней см. в [документации](https://github.com/sirupsen/logrus) к logrus

### Настройки HTTP API (секция `[api.http]`)

|Имя параметра|Тип|Обязательный|Описание|Значение по умолчанию
|---|---|---|---|---|
|addr|string|Да|Адрес http-сервера|-
|shutdown_timeout|duration|Нет|Таймаут ожидания отключения http-сервера|5s
|frontend_directory|string|Нет|Путь к директории frontend части|./frontend

### Настройки мониторинга (секция `[monitoring]`)

|Имя параметра|Тип|Обязательный|Описание|Значение по умолчанию
|---|---|---|---|---|
|timeout|duration|Нет|Таймаут мониторинга|2s

### Настройки обработчика (секция `[processor]`)
#### Настройки обновлений (секция `[processor.updater]`)

|Имя параметра|Тип|Обязательный|Описание|Значение по умолчанию
|---|---|---|---|---|
|symlink_name|string|Нет|Имя symlink-файла, на который ссылается .service файл|trezormon
|service_name|string|Нет|Имя сервиса в systemd|trezormon
|systemd_directory|string|Нет|Локация каталога systemd|/etc/systemd/system/

### Настройки базы данных (секция `[postgres]`)

|Имя параметра|Тип|Обязательный|Описание|Значение по умолчанию
|---|---|---|---|---|
|dsn|string|Да|Строка подключения|-
|connection_timeout|duration|Нет|Таймаут подключения|5s
|max_connections|int|Нет|Максимальное число используемых подключений|25
|init_schema_on_connect|bool|Нет|Обрабатывать схему при подключении|true

<a name="device_descr"></a>
## 3. Описание устройств

### Общие настройки

|Поле|Тип|Обязательный параметр|Описание|Значение по-умолчанию
|---|---|---|---|---|
|id|int64|Да|Идентификатор устройства|Autoincrement
|description|string|Нет|Описание устройства|""
|connection\_timeout|time.Duration(int64)|Нет|Время ожидания подключения к устройству|1000000000
|device\_type|DeviceType(string)|Да|Тип устройства|-
|subdevices|[]Device|Нет|Дочерние устройства(подустройства)|null

Все устройства, с которыми взаимодействует СМ, должны включать в себя данную структуру 

### ICMP

<b>Тип:</b> icmp

<b>Метод работы:</b> icmp эхо-запрос (ping)

<b>Может иметь дочерние устройства</b>: нет

<b>Описание настроек:</b>

|Поле|Тип|Обязательный параметр|Описание|Значение по-умолчанию
|---|---|---|---|---|
|retries_count|int|Нет|Число попыток достучаться до устройства|4
|timeout|time.Duration(int64)|Нет|Интервал попыток запроса|300мс
|addr|string|Да|Адрес устройства|-

<b>Пример json-структуры:</b>

<details><pre><p>{</p><p>        "id": 6,</p><p>        "description": "elemc K2 Controller by ICMP",</p><p>        "connection\_timeout": 1000000000,</p><p>        "device\_type": "icmp",</p><p>        "retries\_count": 2,</p><p>        "timeout": 3000000000,</p><p>        "addr": "home.panov.email"</p><p>}</p></pre></details>

### K2-контроллер

<b>Тип:</b> k2

<b>Метод работы:</b> обращение к HTTP API контроллера

<b>Может иметь дочерние устройства</b>: да (k2_circle)

<b>Автозагрузка дочерних устройств:</b> да

<b>Описание настроек:</b>

|Поле|Тип|Обязательный параметр|Описание|Значение по-умолчанию
|---|---|---|---|---|
|addr|string|Да|Адрес контроллера|-
|username|string|Да|Имя пользователя|-
|password|string|Да|Пароль|-

<b>Пример json-структуры:</b>

<details><pre><p>{</p><p>        "id": 1,</p><p>        "description": "elemc K2 Controller",</p><p>        "connection\_timeout": 1000000000,</p><p>        "device\_type": "k2",</p><p>        "subdevices": [</p><p>            ...</p><p>        ],</p><p>        "addr": "http://somek2controller.com/",</p><p>        "username": "anonymous",</p><p>        "password": "anypassword"</p><p>}</p></pre></details>

### Подустройство K2

<b>Тип:</b> k2-circle

<b>Метод работы:</b> обращение к HTTP API контроллера

<b>Может иметь дочерние устройства</b>: нет

<b>Описание настроек:</b>

|Поле|Тип|Обязательный параметр|Описание|Значение по-умолчанию
|---|---|---|---|---|
|k2\_device\_id|int64|Да|ID устройства в контроллере|-
|k2\_id|string|Да|string\_id устройства в контроллере|-
|k2\_type|string|Да|Тип устройства K2|-
|tag|string|Да|Тэг устройства|-
|addr|string|Да|Адрес контроллера|-
|username|string|Да|Имя пользователя|-
|password|string|Да|Пароль|-

<b>Пример json-структуры:</b>

<details><pre><p>{</p><p>    "id": 5,</p><p>    "description": "",</p><p>    "connection\_timeout": 1000000000,</p><p>    "device\_type": "k2\_circle",</p><p>    "parentID": 1,</p><p>    "k2\_device\_id": 0,</p><p>    "k2\_id": "moxa",</p><p>    "k2\_type": "udp",</p><p>    "tag": "",</p><p>    "addr": "http://somek2controller.com/",</p><p>    "username": "anonymous",</p><p>    "password": "anypassword"</p><p>}</p></pre></details>

## Прочие структуры

<a name="other_plan"></a>
### План

<details><pre><p>{</p><p>    "id": 2,</p><p>    "name": "план1",</p><p>    "description": "тестовый план",</p><p>    "image": "",</p><p>    "max\_scale": 1000,</p><p>    "objects": [</p><p>        {</p><p>            "id": 2,</p><p>            "name": "устройство 1",</p><p>            "type": "device",</p><p>            "obj\_id": 99,</p><p>            "object": {</p><p>                "id": 99,</p><p>                "description": "elemc home",</p><p>                "connection\_timeout": 0,</p><p>                "device\_type": "k2",</p><p>                "last\_state": "Normal",</p><p>                "service\_mode": false,</p><p>                "alarms\_disabled": false,</p><p>                "updated\_at": "0001-01-01T00:00:00+02:30",</p><p>                "created\_at": "2021-03-04T00:10:02.767145+03:00",</p><p>                "last\_state\_at": "2021-03-04T00:10:02.865624+03:00",</p><p>                "addr": "http://home.panov.email/",</p><p>                "username": "admin",</p><p>                "password": "admin"</p><p>            },</p><p>            "locX": 10,</p><p>            "locY": 20,</p><p>            "created\_at": "2021-03-04T00:13:32.29896+03:00",</p><p>            "updated\_at": "2021-03-04T00:15:11.180216+03:00"</p><p>        },</p><p>        {</p><p>            "id": 3,</p><p>            "name": "сегмент 1",</p><p>            "type": "segment",</p><p>            "obj\_id": 6,</p><p>            "object": {</p><p>                "id": 6,</p><p>                "state": "Normal",</p><p>                "name": "test\_segment for timestamps test",</p><p>                "description": "Тестовый сегмент",</p><p>                "devices\_ids": [</p><p>                    99</p><p>                ],</p><p>                "devices": [</p><p>                    {</p><p>                        "id": 99,</p><p>                        "description": "elemc home",</p><p>                        "connection\_timeout": 0,</p><p>                        "device\_type": "k2",</p><p>                        "last\_state": "Normal",</p><p>                        "service\_mode": false,</p><p>                        "alarms\_disabled": false,</p><p>                        "updated\_at": "0001-01-01T00:00:00+02:30",</p><p>                        "created\_at": "2021-03-04T00:10:02.767145+03:00",</p><p>                        "last\_state\_at": "2021-03-04T00:10:02.865624+03:00",</p><p>                        "addr": "http://home.panov.email/",</p><p>                        "username": "admin",</p><p>                        "password": "admin"</p><p>                    }</p><p>                ],</p><p>                "updated\_at": "2021-03-04T00:10:46.159488+03:00",</p><p>                "created\_at": "2021-02-18T22:31:20.428214+03:00",</p><p>                "last\_state\_at": "2021-03-04T00:10:46.201118+03:00"</p><p>            },</p><p>            "locX": 10,</p><p>            "locY": 20,</p><p>            "created\_at": "2021-03-04T00:15:11.181435+03:00",</p><p>            "updated\_at": "0001-01-01T00:00:00+02:30"</p><p>        }</p><p>    ],</p><p>    "created\_at": "2021-03-04T00:13:32.295245+03:00",</p><p>    "updated\_at": "2021-03-04T00:15:11.178327+03:00"</p><p>}</p></pre></details>
